package XXV::MODULES::SVDRP;

use Tools;
use Locale::gettext;
use strict;
use IPC::SysV qw(IPC_PRIVATE IPC_CREAT IPC_RMID SEM_UNDO S_IRWXU);


$|++;

# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'SVDRP',
        Prereq => {
            'IO::Socket::INET'  => 'Object interface for AF_INET domain sockets ',
        },
        Description => gettext('This module a telnet client for sdvrp.'),
        Version => '0.01',
        Date => '07.09.2004',
        Author => 'xpix',
        Preferences => {
            VdrHost => {
                description => gettext('Name of the host that runs vdr'),
                default     => 'localhost',
                type        => 'host',
                required    => gettext('This is required!'),
            },
            VdrPort => {
                description => gettext('SVDRP-port of the running vdr client'),
                default     => 2001,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
        },
        Commands => {
            sstatus => {
                description => gettext('Status from svdrp'),
                short       => 'ss',
                callback    => sub{ $obj->status(@_) },
                Level       => 'user',
            },
            scommand => {
                description => gettext('Send a command to svdrp'),
                short       => 'sc',
                callback    => sub{ $obj->scommand(@_) },
                Level       => 'admin',
            },
        },
    };
    return $args;
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_};
        $self->{$_} = $self->{MOD}->{Preferences}->{$_}->{default} unless($self->{$_});
    } keys %{$self->{MOD}->{Preferences}};

    # Try to use the Requirments
    map {
        eval "use $_";
        return panic("\nCan not load Module: $_\nPlease install this module on your System:\nperl -MCPAN -e 'install $_'") if($@);
    } keys %{$self->{MOD}->{Prereq}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    $self->{COMMANDS} = [];

	return $self;
}

# ------------------
sub connectSocket {
# ------------------
    my $obj = shift || return error ('No Object!' );

    if(! ref $obj->{SOCKET}) {
        lg sprintf('Connect to SVDRP Host: "%s:%d"',
                $obj->{VdrHost},
                $obj->{VdrPort},
            );

    	$obj->{SOCKET} = IO::Socket::INET->new(
    		PeerAddr => $obj->{VdrHost},
    		PeerPort => $obj->{VdrPort},
    		Proto => 'tcp',
            Timeout => 1,
        ) or return error("Can't create Socket: $!");
        return $obj->{SOCKET};
    } else {
        return error('Socket is used from other Process!');
    }
}

# ------------------
sub closeSocket {
# ------------------
    my $obj = shift  || return error ('No Object!' );
    my $socket = shift || $obj->{SOCKET};
    $socket->close();
    $obj->{SOCKET} = 0;
    lg "SVDRP Socket closed ...";
}


# ------------------
sub queue_cmds {
# ------------------
    my $obj = shift  || return error ('No Object!' );
    my $cmd = shift  || 'CALL';

    if($cmd eq 'CALL') {
        my $erg = $obj->command($obj->{COMMANDS});
        $obj->{COMMANDS} = [];
        return $erg;
    } elsif($cmd eq 'COUNT') {
        return scalar @{$obj->{COMMANDS}};
    } else {
        push(@{$obj->{COMMANDS}}, $cmd);
    }
}

# ------------------
sub command {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $cmd = shift || $obj->{COMMANDS} || return error ('No Commands!');
    my $delimiter = shift || '';

    my @commands = ();
    push(@commands, (ref $cmd eq 'ARRAY' ? @$cmd : $cmd));

    # Put Command follow quit and read Output
    my $tosocket;
    foreach my $command (@commands) {
        event(sprintf('Call command "%s" on svdrp', $command));
        $tosocket .= sprintf("%s\r\n", $command);
    }
    $tosocket .= sprintf("%s\r\n", 'quit');

    my $socket = $obj->connectSocket()
        or return error("Couldn't connect svdrp-socket to $obj->{VdrHost}:$obj->{VdrPort}! ($!)");
    print $socket $tosocket;

    my (@Data, $Line);
    while(defined ($Line = <$socket>)) {
        $Line =~ s/\r\n/$delimiter/g;
        push(@Data, $Line);
    };
    $obj->closeSocket();

    my $output = $obj->parse(\@Data, \@commands);

    return $output;
}

# ------------------
sub status {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return;

    my $erg = $obj->command('stat disk');
    $console->msg($erg, $obj->err)
        if(ref $console);
}

# ------------------
sub scommand {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $text = shift || return $console->message(gettext("No Command! Please use scommand 'cmd'"));

    my $erg = $obj->command($text);
    $console->msg($erg, $obj->err);
}

# ------------------
sub parse {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $data = shift || return error ('No Data to parse!');
    my $commands = shift || return error ('No Commands to parse!');

    my @ret;
    my $error = 0;

    main::getVdrVersion($1)
        if($data->[0] =~ /SVDRP\s+VideoDiskRecorder\s+(\d\.\d\.\d+)[\;|\-]/);

    foreach my $zeile (@$data) {
        if($zeile =~ /^(\d{3})\s+(.+)/) {
            $error = $2 if(int($1) >= 500);
        }
    }
    if($error) {
        $obj->{ERROR} = sprintf("The command(s) [%s] produced an error: %s", join(', ', @$commands), $error);
    } else {
        $obj->{ERROR} = 0;
    }
    return $data;
}

# ------------------
sub err {
# ------------------
    my $obj = shift  || return error ('No Object!' );
    return $obj->{ERROR};
}


1;
